@@ -8,7 +8,8 @@ globalG.test <- function(x, listw, zero.policy=FALSE,
 	alternative <- match.arg(alternative, c("greater", "less", "two.sided"))
 	if (!inherits(listw, "listw"))
 	stop(paste(deparse(substitute(listw)), "is not a listw object"))
-	if (listw$style != "B") stop("Only binary weights allowed")
+	if (is.na(match(listw$style, c("B", "C", "U")))) 
+          warning("Binary weights recommended (sepecially for distance bands)")
 	if (!is.numeric(x))
 	stop(paste(deparse(substitute(x)), "is not a numeric vector"))
 	if (any(is.na(x))) stop(paste("NA in ", deparse(substitute(x))))
